
//////////////////////////////////////////////////////////////////////////////
//
// Originally generated by PIDL on Thu Oct 18 21:33:52 2001
//
//////////////////////////////////////////////////////////////////////////////

#include "CTagParse.h"
#include "sptr.h"
#include "debug.h"

#define MIN_START_TAG_LEN 3  //ex: <A>
#define MIN_END_TAG_LEN   4  //ex:</A>


CTagParse::CTagParse(MTagParse* module): m_module(module)
#ifdef PUSH_IUNKNOWN_IMPL_UP
    ,m_CTagParse_refCount(0)
#endif // #ifdef PUSH_IUNKNOWN_IMPL_UP
{
   m_ITagParseEventSink = NULL;
}

CTagParse::~CTagParse()
{ }

//----------------------------------------------------------------------------
// IEventSource
//----------------------------------------------------------------------------

STDMETHODIMP CTagParse::Advise(REFIID iid, void* sinkIntf)
{
    USE_CONTEXT;
    m_ITagParseEventSink = (ITagParseEvent*)sinkIntf;
    return S_OK;
}

STDMETHODIMP CTagParse::Unadvise(void* sinkIntf)
{
    USE_CONTEXT;
    m_ITagParseEventSink = NULL;
    return S_OK;

}


//----------------------------------------------------------------------------
// Non interface methods
//----------------------------------------------------------------------------
BOOL08 CTagParse::VerifyTags(IString *startTag, IString *endTag)
{
#ifdef _DEBUG
   CTraceCall trace("VerifyTags");
#endif
   
   //Verify tags are of the form <TAG></TAG>
   //Simple algorithm:  Extract "/" slash character
   //from endTag and verify two tags are identical

   IPtr<IString, &IID_IString> spTempEndTag;
   U32 foundIndex = 0;
   HRESULT hr;
   
   
   pCoCreateString(L"", &spTempEndTag);

   //At a minimum a tag must consist of at least one
   //character
   if(startTag->GetLength() < MIN_START_TAG_LEN ||
      endTag->GetLength() < MIN_END_TAG_LEN)
      return FALSE;

   //Verify the first tag is of the form <TAG>
   //Verify tag begins with '<'
   hr = startTag->FindFirstOf(L"<", 0, -1, &foundIndex);
   if(hr == S_FALSE)
      return FALSE;

   //Verify there are no other '<' characters
   hr = startTag->FindFirstOf(L"<", 1, -1, &foundIndex);
   if(hr == S_OK)
      return FALSE;

   //Verify tag ends with ">"
   hr = startTag->FindFirstOf(L">", 0, -1, &foundIndex);
   if(hr == S_FALSE)
      return FALSE;
   else
   {
      //if a '>' character occurs before the end,
      //the tag is invalid
      if(foundIndex < (startTag->GetLength()-1))
         return FALSE;
   }

   //Copy the end tag, skipping the forward slash ('/') character.
   spTempEndTag->Append(endTag, 0, 1);
   spTempEndTag->Append(endTag, 2, endTag->GetLength()-2);

   //This temporary end tag should now be identical to the
   //begin tag
   if(startTag->Compare(spTempEndTag, -1) != 0)
      return FALSE;
   
   return TRUE;

}


//----------------------------------------------------------------------------
// ITagParse
//----------------------------------------------------------------------------

STDMETHODIMP CTagParse::ExtractDataByTag(IString* searchStr, IString* startTag, IString* endTag, S32 maxChar, IString*& resultStr)
{
    USE_CONTEXT;
#ifdef _DEBUG
   CTraceCall trace("ExtractDataByTag");
#endif

   U32 ulStartIndex = 0;
   U32 ulEndIndex = 0;
   HRESULT hr;
   
   if(VerifyTags(startTag, endTag) == FALSE)
   {
      if(m_ITagParseEventSink != NULL)
         m_ITagParseEventSink->InvalidTagPair(startTag, endTag);
      return E_INVALIDARG;
   }

   hr = searchStr->FindSubstr(startTag, ulStartIndex, -1, &ulStartIndex);
   if(hr == S_OK)
   {
      ulStartIndex += startTag->GetLength();
      
      hr = searchStr->FindSubstr(endTag, ulStartIndex, -1, &ulEndIndex);
      
      if(hr == S_OK)
      {
         if(ulEndIndex-ulStartIndex <= (U32)maxChar)
            return ExtractDataByIndex(searchStr, ulStartIndex, ulEndIndex-ulStartIndex, resultStr);
         else
            return E_FAIL;
      }
      else
         return E_FAIL;
   }
   else
      return E_FAIL;
   
   return S_OK;

}

STDMETHODIMP CTagParse::ExtractDataByIndex(IString* searchStr, S32 fromIndex, S32 fromCount, IString*& resultStr)
{
    USE_CONTEXT;
#ifdef _DEBUG
   CTraceCall trace("ExtractDataByIndex");
#endif

   HRESULT hr;
   S32 lExtractCount;
   CHAR16 *pData;
   
   IPtr<IHost, &IID_IHost> spHost;

   resultStr = NULL;
   pCoGetHost(&spHost);

   if(fromIndex < 0 || (fromIndex > (S32)(searchStr->GetLength() - 1)))
      return E_FAIL;
   
   //Calculate how much data is available for retrieval
   S32 countAvail = (searchStr->GetLength() - fromIndex);

   fromCount = (fromCount <= countAvail) ? fromCount : countAvail;

   //Calculate many bytes of data need to be allocated for the result
   U32 ulBytesReq = fromCount*(sizeof(CHAR16)) + sizeof(CHAR16);

   if(FAILED(hr = spHost->MemAlloc((void **)&pData, ulBytesReq)))
      return hr;

   spHost->MemSet(pData, 0, ulBytesReq);

   if(FAILED(hr = searchStr->Extract(pData, fromIndex, fromCount, &lExtractCount)))
   {
      spHost->MemFree((void **)&pData);
      return hr;
   }

   if(FAILED(hr = pCoCreateString(pData, &resultStr)))
   {
      spHost->MemFree((void **)&pData);
      return hr;
   }

   spHost->MemFree((void **)&pData);
    return S_OK;

}

STDMETHODIMP CTagParse::CountTagPairs(IString* searchStr, IString* startTag, IString* endTag, S32& numItems)
{
    USE_CONTEXT;
#ifdef _DEBUG
   CTraceCall trace("CountTagPairs");
#endif

   HRESULT hr;
   numItems = 0;
   BOOL08 bContinue = TRUE;
   U32 ulFoundIndex = 0;
   U32 ulStartIndex = 0;

   Debug("searchStr = "); Debugln(searchStr);
   Debug("startTag = "); Debugln(startTag);
   Debug("endTag = "); Debugln(endTag);
      
   if(VerifyTags(startTag, endTag) == FALSE)
   {
      if(m_ITagParseEventSink != NULL)
         m_ITagParseEventSink->InvalidTagPair(startTag, endTag);
      return S_OK;
   }
   
   while(bContinue == TRUE)
   {
      //Find start tag
      hr = searchStr->FindSubstr(startTag, ulFoundIndex, -1, &ulFoundIndex);

      if(hr == S_OK)
      {
         //increment ulFoundIndex past the start tag
         ulFoundIndex += startTag->GetLength();

         //Find the end tag
         hr = searchStr->FindSubstr(endTag, ulFoundIndex, -1, &ulFoundIndex);
         if(hr == S_OK)
         {
           //we have found both the start and end tags
           numItems++;
           
           //increment past the end tag
           ulFoundIndex += endTag->GetLength();
         }
         else
         {
            bContinue = FALSE;
            //if we found a start tag, we should find a corresponding end tag
            if(m_ITagParseEventSink != NULL)
               m_ITagParseEventSink->TagNotFound(endTag);
         }

      }
      else  //There are no (more) tag pairs
         bContinue = FALSE;

   }
   return S_OK;

}


// Always include the generated file last
#include "_CTagParse.inc"

